@@ -53,7 +53,10 @@ impl HdfsFs {
     /// Create instance of HdfsFs with a global cache.
     /// For each namenode uri, only one instance will be created
     pub fn new(path: &str) -> Result<HdfsFs, HdfsErr> {
-        let namenode_uri = get_namenode_uri(path)?;
+        let namenode_uri = match path {
+            "default" => String::from("default"),
+            _ => get_namenode_uri(path)?,
+        };
 
         {
             // Get if exists
@@ -92,12 +95,33 @@ impl HdfsFs {
         };
     }
 
+    /// The default NameNode configuration will be used (from the XML configuration files)
+    pub fn default() -> Result<HdfsFs, HdfsErr> {
+        HdfsFs::new("default")
+    }
+
+    pub fn remove(path: &str) -> Result<Option<HdfsFs>, HdfsErr> {
+        let namenode_uri = match path {
+            "default" => String::from("default"),
+            _ => get_namenode_uri(path)?,
+        };
+
+        let mut cache = HDFS_CACHE.write().unwrap();
+        Ok(cache.remove(&namenode_uri))
+    }
+
     /// Get HDFS namenode url
     #[inline]
     pub fn url(&self) -> &str {
         &self.url
     }
 
+    /// Get a raw pointer of JNI API's HdfsFs
+    #[inline]
+    pub fn raw(&self) -> hdfsFS {
+        self.raw
+    }
+
     /// Create HdfsFile from hdfsFile
     fn new_hdfs_file(&self, path: &str, file: hdfsFile) -> Result<HdfsFile, HdfsErr> {
         if file.is_null() {
@@ -683,8 +707,8 @@ impl Drop for BlockHosts {
     }
 }
 
-const LOCAL_FS_SCHEME: &'static str = "file";
-const HDFS_FS_SCHEME: &'static str = "hdfs";
+pub const LOCAL_FS_SCHEME: &'static str = "file";
+pub const HDFS_FS_SCHEME: &'static str = "hdfs";
 
 #[inline]
 fn get_namenode_uri(path: &str) -> Result<String, HdfsErr> {
@@ -710,65 +734,115 @@ fn get_namenode_uri(path: &str) -> Result<String, HdfsErr> {
     }
 }
 
+#[inline]
+pub fn get_uri(path: &str) -> Result<String, HdfsErr> {
+    let path = if path.starts_with("/") {
+        format!("{}://{}", LOCAL_FS_SCHEME, path)
+    } else {
+        path.to_string()
+    };
+    match Url::parse(&path) {
+        Ok(url) => match url.scheme() {
+            LOCAL_FS_SCHEME | HDFS_FS_SCHEME => Ok(url.to_string()),
+            _ => Err(HdfsErr::InvalidUrl(path.to_string())),
+        },
+        Err(_) => Err(HdfsErr::InvalidUrl(path.to_string())),
+    }
+}
+
 #[cfg(test)]
 mod test {
-    use crate::hdfs::HdfsFs;
-    use crate::test::run_hdfs_test;
+    use crate::minidfs::get_dfs;
+    use uuid::Uuid;
+
+    #[cfg(feature = "use_existing_hdfs")]
+    #[test]
+    fn test_hdfs_default() {
+        let fs = super::HdfsFs::default().ok().unwrap();
+
+        let uuid = Uuid::new_v4().to_string();
+        let test_file = uuid.as_str();
+        let created_file = match fs.create(test_file) {
+            Ok(f) => f,
+            Err(_) => panic!("Couldn't create a file"),
+        };
+        assert!(created_file.close().is_ok());
+        assert!(fs.exist(test_file));
+
+        assert!(fs.delete(test_file, false).is_ok());
+        assert!(!fs.exist(test_file));
+    }
 
     #[test]
     fn test_hdfs() {
-        run_hdfs_test(|dfs| {
-            let port = dfs.namenode_port().unwrap();
-            let minidfs_addr = format!("hdfs://localhost:{}", port);
+        let dfs = get_dfs();
+        {
+            let minidfs_addr = dfs.namenode_addr();
+            let fs = dfs.get_hdfs().ok().unwrap();
 
-            // create a file, check existence, and close
-            let fs = HdfsFs::new(minidfs_addr.as_str()).ok().unwrap();
-            let test_file = "/test_file";
-            let created_file = match fs.create(test_file) {
-                Ok(f) => f,
-                Err(_) => panic!("Couldn't create a file"),
-            };
-            assert!(created_file.close().is_ok());
-            assert!(fs.exist(test_file));
-
-            // open a file and close
-            let opened_file = fs.open(test_file).ok().unwrap();
-            assert!(opened_file.close().is_ok());
-
-            match fs.mkdir("/dir1") {
-                Ok(_) => println!("/dir1 created"),
-                Err(_) => panic!("Couldn't create /dir1 directory"),
-            };
-
-            let file_info = fs.get_file_status("/dir1").ok().unwrap();
-
-            let expected_path = format!("hdfs://localhost:{}/dir1", port);
-            assert_eq!(&expected_path, file_info.name());
-            assert!(!file_info.is_file());
-            assert!(file_info.is_directory());
-
-            let sub_dir_num = 3;
-            let mut expected_list = Vec::new();
-            for x in 0..sub_dir_num {
-                let filename = format!("/dir1/{}", x);
-                expected_list.push(format!("hdfs://localhost:{}/dir1/{}", port, x));
-
-                match fs.mkdir(&filename) {
-                    Ok(_) => println!("/dir1.x created"),
-                    Err(_) => panic!("Couldn't create /dir1 directory"),
-                };
-            }
-
-            let mut list = fs.list_status("/dir1").ok().unwrap();
-            assert_eq!(sub_dir_num, list.len());
-
-            list.sort_by(|a, b| Ord::cmp(a.name(), b.name()));
-            for (expected, name) in expected_list
-                .iter()
-                .zip(list.iter().map(|status| status.name()))
+            // Test hadoop file
             {
-                assert_eq!(expected, name);
+                // create a file, check existence, and close
+                let uuid = Uuid::new_v4().to_string();
+                let test_file = uuid.as_str();
+                let created_file = match fs.create(test_file) {
+                    Ok(f) => f,
+                    Err(_) => panic!("Couldn't create a file"),
+                };
+                assert!(created_file.close().is_ok());
+                assert!(fs.exist(test_file));
+
+                // open a file and close
+                let opened_file = fs.open(test_file).ok().unwrap();
+                assert!(opened_file.close().is_ok());
+
+                // Clean up
+                assert!(fs.delete(test_file, false).is_ok());
             }
-        })
+
+            // Test directory
+            {
+                let uuid = Uuid::new_v4().to_string();
+                let test_dir = format!("/{}", uuid);
+
+                match fs.mkdir(&test_dir) {
+                    Ok(_) => println!("{} created", test_dir),
+                    Err(_) => panic!("Couldn't create {} directory", test_dir),
+                };
+
+                let file_info = fs.get_file_status(&test_dir).ok().unwrap();
+
+                let expected_path = format!("{}{}", minidfs_addr, test_dir);
+                assert_eq!(&expected_path, file_info.name());
+                assert!(!file_info.is_file());
+                assert!(file_info.is_directory());
+
+                let sub_dir_num = 3;
+                let mut expected_list = Vec::new();
+                for x in 0..sub_dir_num {
+                    let filename = format!("{}/{}", test_dir, x);
+                    expected_list.push(format!("{}{}/{}", minidfs_addr, test_dir, x));
+
+                    match fs.mkdir(&filename) {
+                        Ok(_) => println!("{} created", filename),
+                        Err(_) => panic!("Couldn't create {} directory", filename),
+                    };
+                }
+
+                let mut list = fs.list_status(&test_dir).ok().unwrap();
+                assert_eq!(sub_dir_num, list.len());
+
+                list.sort_by(|a, b| Ord::cmp(a.name(), b.name()));
+                for (expected, name) in expected_list
+                    .iter()
+                    .zip(list.iter().map(|status| status.name()))
+                {
+                    assert_eq!(expected, name);
+                }
+
+                // Clean up
+                assert!(fs.delete(&test_dir, true).is_ok());
+            }
+        }
     }
 }
